// ============================================================================
// ==                                                                        ==
// == Amdis - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ArhReader.h */

#ifndef AMDIS_ARH_READER_H
#define AMDIS_ARH_READER_H

#include "AMDiS_fwd.h"

namespace AMDiS {

  using namespace std;

  class ArhReader
  {
  public:
    static void read(string filename, Mesh *mesh,
		     DOFVector<double>* vec0 = NULL, 
		     DOFVector<double>* vec1 = NULL,
		     DOFVector<double>* vec2 = NULL,		     
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
		     bool writeParallel = true,
#else
		     bool writeParallel = false,
#endif
		     int nProcs = -1);

    static void read(string filename, Mesh *mesh, 
		     vector<DOFVector<double>*> vecs,
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
		     bool writeParallel = true,
#else
		     bool writeParallel = false,
#endif
		     int nProcs = -1);

    static int getNumValueVectors(string filename)
    {    
      ifstream file;
      file.open(filename.c_str(), ios::in | ios::binary);

      string typeId = "";
      uint32_t nMacroElements = 0;
      uint32_t nValueVectors = 0;
      uint32_t nAllValues = 0;

      file.read(const_cast<char*>(typeId.data()), 4);
      file.read(reinterpret_cast<char*>(&nMacroElements), 4);
      file.read(reinterpret_cast<char*>(&nValueVectors), 4);
      file.read(reinterpret_cast<char*>(&nAllValues), 4);

      file.close();
      
      return nValueVectors;
    }
    
  private:

    static void readFile(string filename, 
			 Mesh *mesh,
			 vector<DOFVector<double>*> vecs);

    static void setDofValues(int macroElIndex, Mesh *mesh, 
			     vector<double>& values, DOFVector<double>* vec);
  };
}

#endif
